
Macro CNDq
  "
EndMacro

Macro CNIsList(_container_)
  Bool(CNDq#_container_#CNDq = "List")
EndMacro


#CNMinimalArraySize = 16


Macro InitListContainer(_type_)
  CompilerIf Not Defined(ListContainer#_type_, #PB_Structure)
    
    Structure ListContainer#_type_
      *VTable
      List elements._type_()
    EndStructure
    
  CompilerEndIf
EndMacro


Macro InitArrayContainer(_type_)
  CompilerIf Not Defined(ArrayContainer#_type_, #PB_Structure)
    
    Structure ArrayContainer#_type_
      *VTable
      _number_.i
      _current_.i
      Array elements._type_(#CNMinimalArraySize-1)
    EndStructure
    
    
    ;- ArrayContainer_add
    Procedure ArrayContainer#_type_#_add(*this.ArrayContainer#_type_)
      Protected i
      
      If (*this\_number_ > ArraySize(*this\elements()))
        ReDim *this\elements(*this\_number_ * 2 - 1)
      EndIf
      
      If (*this\_number_ > 0) Or (*this\_current_ = -1)
        *this\_current_ + 1
      EndIf
      
      If (*this\_current_ < *this\_number_)
        For i = *this\_number_ - 1 To *this\_current_ Step -1
          *this\elements(i+1) = *this\elements(i)
        Next
        
        ClearStructure(*this\elements(*this\_current_), _type_)
      EndIf
      
      *this\_number_ + 1
      InitializeStructure(*this\elements(*this\_current_), _type_)
      
      ProcedureReturn *this\elements(*this\_current_)
    EndProcedure
    
    
    ;- ArrayContainer_del
    Procedure ArrayContainer#_type_#_del(*this.ArrayContainer#_type_)
      Protected i
      
      If (*this\_current_ >= 0) And (*this\_number_ > 0)
        
        If (*this\_current_ < *this\_number_-1)
          For i = *this\_current_ To *this\_number_ - 2
            *this\elements(i) = *this\elements(i+1)
          Next
        EndIf
        
        ClearStructure(*this\elements(*this\_number_-1), _type_)
        
        If (*this\_number_ >= #CNMinimalArraySize-8) And (*this\_number_ - 1 < (ArraySize(*this\elements())+1)/2-8)
          ReDim *this\elements((ArraySize(*this\elements())+1)/2-1)
        EndIf
        
        *this\_number_ - 1
        *this\_current_ - 1
      EndIf
    EndProcedure
    
    
    ;- ArrayContainer_clear
    Procedure ArrayContainer#_type_#_clear(*this.ArrayContainer#_type_)
      Dim *this\elements(#CNMinimalArraySize-1)
      *this\_number_  = 0
      *this\_current_ = -1
    EndProcedure
    
  CompilerEndIf
EndMacro


Macro InitContainer(_type_, _container_=List)
  CompilerIf CNIsList(_container_)
    InitListContainer(_type_)
    
  CompilerElse
    InitArrayContainer(_type_)
    
  CompilerEndIf
EndMacro


Macro CNStructure(_type_, _container_=List)
  _container_#Container#_type_
EndMacro


Macro CNSize(_res_, _container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    _res_ = ListSize(_this_\elements())
    
  CompilerElse
    _res_ = _this_\_number_
    
  CompilerEndIf
EndMacro


Macro CNReset(_container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    ResetList(_this_\elements())
    
  CompilerElse
    _this_\_current_ = -1
    
  CompilerEndIf
EndMacro


Macro CNAdd(_res_, _type_, _container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    _res_ = AddElement(_this_\elements())
    
  CompilerElse
    _res_ = ArrayContainer#_type_#_add(_this_)
    
  CompilerEndIf
EndMacro


Macro CNDel(_type_, _container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    DeleteElement(_this_\elements())
    
  CompilerElse
    ArrayContainer#_type_#_del(_this_)
    
  CompilerEndIf
EndMacro


Macro CNClear(_type_, _container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    ClearList(_this_\elements())
    
  CompilerElse
    ArrayContainer#_type_#_clear(_this_)
    
  CompilerEndIf
EndMacro


Macro CNGet(_res_, _container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    _res_ = @_this_\elements()
    
  CompilerElse
    If (_this_\_current_ >= 0) And (_this_\_number_ > 0)
      _res_ = _this_\elements(_this_\_current_)
    Else
      _res_ = #Null
    EndIf
    
  CompilerEndIf
EndMacro


Macro CNNext(_res_, _container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    _res_ = NextElement(_this_\elements())
    
  CompilerElse
    If (_this_\_current_ < _this_\_number_-1)
      _this_\_current_ + 1
      CNGet(_res_, _container_, _this_)
    Else
      _res_ = #Null
    EndIf
    
  CompilerEndIf
EndMacro


Macro CNPrev(_res_, _container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    _res_ = PreviousElement(_this_\elements())
    
  CompilerElse
    If (_this_\_current_ > 0)
      _this_\_current_ - 1
      CNGet(_res_, _container_, _this_)
    Else
      _res_ = #Null
    EndIf
    
  CompilerEndIf
EndMacro


Macro CNFirst(_container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    FirstElement(_this_\elements())
    
  CompilerElse
    _this_\_current_ = 0
    
  CompilerEndIf
EndMacro


Macro CNLast(_container_=List, _this_=*this)
  CompilerIf CNIsList(_container_)
    LastElement(_this_\elements())
    
  CompilerElse
    _this_\_current_ = _this_\_number_-1
    
  CompilerEndIf
EndMacro


Structure Void
EndStructure


Prototype Comparator(*first, *second)


Procedure ComparatorInteger(*first.Integer, *second.Integer)
  ProcedureReturn *first\i - *second\i
EndProcedure


Procedure ComparatorDouble(*first.Double, *second.Double)
  If (*first\d < *second\d)
    ProcedureReturn -1
  ElseIf (*first\d > *second\d)
    ProcedureReturn 1
  Else
    ProcedureReturn 0
  EndIf
EndProcedure


Procedure ComparatorString(*first.String, *second.String)
  ProcedureReturn CompareMemoryString(@*first\s, @*second\s)
EndProcedure


; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; Folding = ----
; EnableXP