
XIncludeFile "..\TreeMap.pb"
XIncludeFile "..\TreeSet.pb"


Structure Car
  Weight.l
  Speed.l
  Price.l
EndStructure


; initialize the structures and procedures of TreeSet with type Integer
InitTreeSet(Integer)

; initialize the structures and procedures of TreeMap
; with key-type String and value-type Integer
InitTreeMap(String, Integer)

; initialize the structures and procedures of TreeMap
; with key-type String and value-type Car
InitTreeMap(String, Car)

; initialize the structures and procedures of TreeMap
; with key-type String and value-type TreeSetInteger
; TreeSetInteger is the structure of TreeSet with type Integer
InitTreeMap(Integer, TreeSetInteger)


Procedure AllocateTreeMap()
  Protected *map.TreeMap
  
  *map = NewTreeMap(String, Car, @ComparatorString())
  
  ; *map\put(...)
  ; ...
  
  *map\free()
EndProcedure


Procedure UseLocalTreeMap()
  Protected local.TreeMapStringCar
  Protected *map.TreeMap
  
  *map = NewTreeMap(String, Car, @ComparatorString(), local)
  
  ; *map\put(...)
  ; ...
  
  ; *map\free() is not needed
EndProcedure


Procedure TreeMapWithCars()
  Protected *map.TreeMap
  Protected key.String
  Protected *key.String, *car.Car
  
  *map = NewTreeMap(String, Car, @ComparatorString())
  
  key\s = "Ferrari F40"
  *car = *map\put(key)
  *car\Weight = 1000
  *car\Speed = 320
  *car\Price = 500000
  
  key\s = "Lamborghini Gallardo"
  *car = *map\put(key)
  *car\Weight = 1200
  *car\Speed = 340
  *car\Price = 700000
  
  ; iterate over keys
  *map\reset()
  While *map\nextKey()
    *key = *map\getCurrentKey()
    Debug *key\s
  Wend
  Debug "--------"
  
  ; nextEntry() and nextValue() without parameters is equivalent to nextKey()
  *map\reset()
  While *map\nextValue()
    *key = *map\getCurrentKey()
    *car = *map\getCurrentValue()
    Debug *key\s
    Debug *car\Price
  Wend
  Debug "--------"
  
  ; the optional parameter can be used to get the current
  ; entry/key/value without the getCurrentXX()-method
  *map\reset()
  While *map\nextKey(@*key)
    Debug *key\s
  Wend
  Debug "--------"
  
  *map\reset()
  While *map\nextValue(@*car)
    Debug *car\Weight
  Wend
  Debug "--------"
  
  *map\free()
EndProcedure


Procedure TreeMapWithInteger()
  Protected *map.TreeMap
  Protected *value.Integer
  Protected *entry.TreeMapEntry
  Protected key.String
  Protected int.i, isAdded.i
  
  *map = NewTreeMap(String, Integer, @ComparatorString())
  
  ; put and set value after that
  key\s = "A"
  *value = *map\put(key)
  *value\i = 1
  
  ; set value before
  key\s = "B"
  int = 2
  *map\put(key, @int)
  
  ; put and check whether key was already contained
  key\s = "C"
  *value = *map\put(key, #Null, @isAdded)
  *value\i = 4
  Debug "Is c added: " + isAdded
  
  ; override the value, but isAdded will be false (0)
  key\s = "C"
  int = 3
  *map\put(key, @int, @isAdded)
  Debug "Is c added: " + isAdded
  
  ; key "C" is already contained, the value will not be overritten
  key\s = "C"
  int = 42
  *map\putIfAbsent(key, @int, @isAdded)
  
  ; get the value 3
  key\s = "C"
  *value = *map\getValue(key)
  Debug "GetValue: " + *value\i
  
  ; the key "D" is not contained, get the default value 4
  key\s = "D"
  int = 4
  Debug "GetOrDefault: " + PeekI(*map\getOrDefault(key, @int))
  
  ; delete the entry with key "B"
  key\s = "B"
  *map\delete(key)
  
  *map\reset()
  While *map\nextEntry(@*entry)
    
    ; you can also use '*map\getCurrentValue()'
    If (PeekI(*entry\getValue()) = 1)
      ; 'deleteEntry' O(1) is more efficient than 'delete' O(log(n)),
      ; but it needs the pointer to the entry
      *map\deleteEntry(*entry)
    EndIf
  Wend
  
  Debug "--------"
  
  *map\free()
EndProcedure


Procedure TreeMapWithTreeSets()
  Protected *map.TreeMap
  Protected *set.TreeSet
  Protected i, k, first, str.s
  
  *map = NewTreeMap(Integer, TreeSetInteger, @ComparatorInteger())
  
  For i = 1 To 5
    *set = *map\put(@i)
    *set = NewTreeSet(Integer, @ComparatorInteger(), *set)
    
    For k = 1 To i-1
      *set\add(@k)
    Next
  Next
  
  *map\reset()
  While *map\nextEntry()
    str = "Key: " + PeekI(*map\getCurrentKey()) + ", Set: {"
    
    first = #True
    *set = *map\getCurrentValue()
    *set\reset()
    
    While *set\nextEl()
      If (first = #False)
        str + ", "
      EndIf
      first = #False
      
      str + PeekI(*set\getCurrent())
    Wend
    
    str + "}"
    Debug str
  Wend
  
  *map\free()
EndProcedure


AllocateTreeMap()
UseLocalTreeMap()
TreeMapWithCars()
TreeMapWithInteger()
TreeMapWithTreeSets()


; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; Folding = -
; EnableXP